@@ -23,18 +23,18 @@ struct Config {
 };
 
 struct autostart : Config<bool> {
-    static QString name() { return "autostart"; }
+    static QString name() { return QStringLiteral("autostart"); }
     static Value defaultValue();
 };
 
 struct maxitems : Config<int> {
-    static QString name() { return "maxitems"; }
+    static QString name() { return QStringLiteral("maxitems"); }
     static Value defaultValue() { return 200; }
     static Value value(Value v) { return qBound(0, v, maxItems); }
 };
 
 struct item_data_threshold : Config<int> {
-    static QString name() { return "item_data_threshold"; }
+    static QString name() { return QStringLiteral("item_data_threshold"); }
     static Value defaultValue() { return 1024; }
     static const char *description() {
         return "Maximum item data size in bytes to save in tab data file"
@@ -43,93 +43,93 @@ struct item_data_threshold : Config<int> {
 };
 
 struct clipboard_tab : Config<QString> {
-    static QString name() { return "clipboard_tab"; }
+    static QString name() { return QStringLiteral("clipboard_tab"); }
     static Value defaultValue() { return defaultClipboardTabName(); }
 };
 
 struct expire_tab : Config<int> {
-    static QString name() { return "expire_tab"; }
+    static QString name() { return QStringLiteral("expire_tab"); }
 };
 
 struct editor : Config<QString> {
-    static QString name() { return "editor"; }
+    static QString name() { return QStringLiteral("editor"); }
     static Value defaultValue();
 };
 
 struct item_popup_interval : Config<int> {
-    static QString name() { return "item_popup_interval"; }
+    static QString name() { return QStringLiteral("item_popup_interval"); }
 };
 
 struct notification_position : Config<int> {
-    static QString name() { return "notification_position"; }
+    static QString name() { return QStringLiteral("notification_position"); }
     static Value defaultValue() { return 3; }
 };
 
 struct clipboard_notification_lines : Config<int> {
-    static QString name() { return "clipboard_notification_lines"; }
+    static QString name() { return QStringLiteral("clipboard_notification_lines"); }
     static Value value(Value v) { return qBound(0, v, 100); }
 };
 
 struct notification_horizontal_offset : Config<int> {
-    static QString name() { return "notification_horizontal_offset"; }
+    static QString name() { return QStringLiteral("notification_horizontal_offset"); }
     static Value defaultValue() { return 10; }
 };
 
 struct notification_vertical_offset : Config<int> {
-    static QString name() { return "notification_vertical_offset"; }
+    static QString name() { return QStringLiteral("notification_vertical_offset"); }
     static Value defaultValue() { return 10; }
 };
 
 struct notification_maximum_width : Config<int> {
-    static QString name() { return "notification_maximum_width"; }
+    static QString name() { return QStringLiteral("notification_maximum_width"); }
     static Value defaultValue() { return 300; }
 };
 
 struct notification_maximum_height : Config<int> {
-    static QString name() { return "notification_maximum_height"; }
+    static QString name() { return QStringLiteral("notification_maximum_height"); }
     static Value defaultValue() { return 100; }
 };
 
 struct edit_ctrl_return : Config<bool> {
-    static QString name() { return "edit_ctrl_return"; }
+    static QString name() { return QStringLiteral("edit_ctrl_return"); }
     static Value defaultValue() { return true; }
 };
 
 struct move : Config<bool> {
-    static QString name() { return "move"; }
+    static QString name() { return QStringLiteral("move"); }
     static Value defaultValue() { return true; }
 };
 
 struct show_simple_items : Config<bool> {
-    static QString name() { return "show_simple_items"; }
+    static QString name() { return QStringLiteral("show_simple_items"); }
     static Value defaultValue() { return false; }
 };
 
 struct number_search : Config<bool> {
-    static QString name() { return "number_search"; }
+    static QString name() { return QStringLiteral("number_search"); }
     static Value defaultValue() { return false; }
 };
 
 struct check_clipboard : Config<bool> {
-    static QString name() { return "check_clipboard"; }
+    static QString name() { return QStringLiteral("check_clipboard"); }
     static Value defaultValue() { return true; }
 };
 
 struct confirm_exit : Config<bool> {
-    static QString name() { return "confirm_exit"; }
+    static QString name() { return QStringLiteral("confirm_exit"); }
     static Value defaultValue() { return true; }
 };
 
 struct vi : Config<bool> {
-    static QString name() { return "vi"; }
+    static QString name() { return QStringLiteral("vi"); }
 };
 
 struct save_filter_history : Config<bool> {
-    static QString name() { return "save_filter_history"; }
+    static QString name() { return QStringLiteral("save_filter_history"); }
 };
 
 struct filter_regular_expression : Config<bool> {
-    static QString name() { return "filter_regular_expression"; }
+    static QString name() { return QStringLiteral("filter_regular_expression"); }
     static Value defaultValue() { return false; }
     static const char *description() {
         return "Use regular expressions to search items";
@@ -137,7 +137,7 @@ struct filter_regular_expression : Config<bool> {
 };
 
 struct filter_case_insensitive : Config<bool> {
-    static QString name() { return "filter_case_insensitive"; }
+    static QString name() { return QStringLiteral("filter_case_insensitive"); }
     static Value defaultValue() { return true; }
     static const char *description() {
         return "Use case-insensitive item search";
@@ -145,16 +145,16 @@ struct filter_case_insensitive : Config<bool> {
 };
 
 struct always_on_top : Config<bool> {
-    static QString name() { return "always_on_top"; }
+    static QString name() { return QStringLiteral("always_on_top"); }
 };
 
 struct close_on_unfocus : Config<bool> {
-    static QString name() { return "close_on_unfocus"; }
+    static QString name() { return QStringLiteral("close_on_unfocus"); }
     static Value defaultValue() { return true; }
 };
 
 struct close_on_unfocus_delay_ms : Config<int> {
-    static QString name() { return "close_on_unfocus_delay_ms"; }
+    static QString name() { return QStringLiteral("close_on_unfocus_delay_ms"); }
     static Value defaultValue() { return 500; }
     static const char *description() {
         return "Delay to close the main window when unfocused/deactivated";
@@ -162,12 +162,12 @@ struct close_on_unfocus_delay_ms : Config<int> {
 };
 
 struct open_windows_on_current_screen : Config<bool> {
-    static QString name() { return "open_windows_on_current_screen"; }
+    static QString name() { return QStringLiteral("open_windows_on_current_screen"); }
     static Value defaultValue() { return true; }
 };
 
 struct restore_geometry : Config<bool> {
-    static QString name() { return "restore_geometry"; }
+    static QString name() { return QStringLiteral("restore_geometry"); }
     static Value defaultValue() { return true; }
     static const char *description() {
         return "Restore position and size for the main window and other dialogs";
@@ -175,38 +175,38 @@ struct restore_geometry : Config<bool> {
 };
 
 struct transparency_focused : Config<int> {
-    static QString name() { return "transparency_focused"; }
+    static QString name() { return QStringLiteral("transparency_focused"); }
     static Value value(Value v) { return qBound(0, v, 100); }
 };
 
 struct transparency : Config<int> {
-    static QString name() { return "transparency"; }
+    static QString name() { return QStringLiteral("transparency"); }
     static Value value(Value v) { return qBound(0, v, 100); }
 };
 
 struct hide_tabs : Config<bool> {
-    static QString name() { return "hide_tabs"; }
+    static QString name() { return QStringLiteral("hide_tabs"); }
 };
 
 struct hide_toolbar : Config<bool> {
-    static QString name() { return "hide_toolbar"; }
+    static QString name() { return QStringLiteral("hide_toolbar"); }
 };
 
 struct hide_toolbar_labels : Config<bool> {
-    static QString name() { return "hide_toolbar_labels"; }
+    static QString name() { return QStringLiteral("hide_toolbar_labels"); }
     static Value defaultValue() { return true; }
 };
 
 struct disable_tray : Config<bool> {
-    static QString name() { return "disable_tray"; }
+    static QString name() { return QStringLiteral("disable_tray"); }
 };
 
 struct hide_main_window : Config<bool> {
-    static QString name() { return "hide_main_window"; }
+    static QString name() { return QStringLiteral("hide_main_window"); }
 };
 
 struct hide_main_window_in_task_bar : Config<bool> {
-    static QString name() { return "hide_main_window_in_task_bar"; }
+    static QString name() { return QStringLiteral("hide_main_window_in_task_bar"); }
     static Value defaultValue() { return false; }
     static const char *description() {
         return "Avoid showing entry for the main window in the task bar";
@@ -214,20 +214,20 @@ struct hide_main_window_in_task_bar : Config<bool> {
 };
 
 struct tab_tree : Config<bool> {
-    static QString name() { return "tab_tree"; }
+    static QString name() { return QStringLiteral("tab_tree"); }
 };
 
 struct show_tab_item_count : Config<bool> {
-    static QString name() { return "show_tab_item_count"; }
+    static QString name() { return QStringLiteral("show_tab_item_count"); }
 };
 
 struct text_wrap : Config<bool> {
-    static QString name() { return "text_wrap"; }
+    static QString name() { return QStringLiteral("text_wrap"); }
     static Value defaultValue() { return true; }
 };
 
 struct text_tab_width : Config<int> {
-    static QString name() { return "text_tab_width"; }
+    static QString name() { return QStringLiteral("text_tab_width"); }
     static Value defaultValue() { return 8; }
     static const char *description() {
         return "Width of Tab character in number of spaces";
@@ -235,53 +235,53 @@ struct text_tab_width : Config<int> {
 };
 
 struct activate_item_with_single_click : Config<bool> {
-    static QString name() { return "activate_item_with_single_click"; }
+    static QString name() { return QStringLiteral("activate_item_with_single_click"); }
     static Value defaultValue() { return false; }
 };
 
 struct activate_closes : Config<bool> {
-    static QString name() { return "activate_closes"; }
+    static QString name() { return QStringLiteral("activate_closes"); }
     static Value defaultValue() { return true; }
 };
 
 struct activate_focuses : Config<bool> {
-    static QString name() { return "activate_focuses"; }
+    static QString name() { return QStringLiteral("activate_focuses"); }
     static Value defaultValue() { return true; }
 };
 
 struct activate_pastes : Config<bool> {
-    static QString name() { return "activate_pastes"; }
+    static QString name() { return QStringLiteral("activate_pastes"); }
     static Value defaultValue() { return true; }
 };
 
 struct tray_items : Config<int> {
-    static QString name() { return "tray_items"; }
+    static QString name() { return QStringLiteral("tray_items"); }
     static Value defaultValue() { return 5; }
     static Value value(Value v) { return qBound(0, v, 99); }
 };
 
 struct tray_item_paste : Config<bool> {
-    static QString name() { return "tray_item_paste"; }
+    static QString name() { return QStringLiteral("tray_item_paste"); }
     static Value defaultValue() { return true; }
 };
 
 struct tray_commands : Config<bool> {
-    static QString name() { return "tray_commands"; }
+    static QString name() { return QStringLiteral("tray_commands"); }
     static Value defaultValue() { return true; }
 };
 
 struct tray_tab_is_current : Config<bool> {
-    static QString name() { return "tray_tab_is_current"; }
+    static QString name() { return QStringLiteral("tray_tab_is_current"); }
     static Value defaultValue() { return true; }
 };
 
 struct tray_images : Config<bool> {
-    static QString name() { return "tray_images"; }
+    static QString name() { return QStringLiteral("tray_images"); }
     static Value defaultValue() { return true; }
 };
 
 struct tray_menu_open_on_left_click : Config<bool> {
-    static QString name() { return "tray_menu_open_on_left_click"; }
+    static QString name() { return QStringLiteral("tray_menu_open_on_left_click"); }
     static Value defaultValue() { return false; }
     static const char *description() {
         return "Open tray menu on left mouse button";
@@ -289,11 +289,11 @@ struct tray_menu_open_on_left_click : Config<bool> {
 };
 
 struct tray_tab : Config<QString> {
-    static QString name() { return "tray_tab"; }
+    static QString name() { return QStringLiteral("tray_tab"); }
 };
 
 struct command_history_size : Config<int> {
-    static QString name() { return "command_history_size"; }
+    static QString name() { return QStringLiteral("command_history_size"); }
     static Value defaultValue() { return 100; }
     static const char *description() {
         return "Number of commands to keep in action dialog history";
@@ -301,24 +301,24 @@ struct command_history_size : Config<int> {
 };
 
 struct check_selection : Config<bool> {
-    static QString name() { return "check_selection"; }
+    static QString name() { return QStringLiteral("check_selection"); }
 };
 
 struct copy_clipboard : Config<bool> {
-    static QString name() { return "copy_clipboard"; }
+    static QString name() { return QStringLiteral("copy_clipboard"); }
 };
 
 struct copy_selection : Config<bool> {
-    static QString name() { return "copy_selection"; }
+    static QString name() { return QStringLiteral("copy_selection"); }
 };
 
 struct run_selection : Config<bool> {
-    static QString name() { return "run_selection"; }
+    static QString name() { return QStringLiteral("run_selection"); }
     static Value defaultValue() { return true; }
 };
 
 struct tabs : Config<QStringList> {
-    static QString name() { return "tabs"; }
+    static QString name() { return QStringLiteral("tabs"); }
     static Value value(Value v) {
         v.removeAll(QString());
         v.removeDuplicates();
@@ -330,7 +330,7 @@ struct tabs : Config<QStringList> {
 };
 
 struct show_advanced_command_settings : Config<bool> {
-    static QString name() { return "show_advanced_command_settings"; }
+    static QString name() { return QStringLiteral("show_advanced_command_settings"); }
     static Value defaultValue() { return false; }
     static const char *description() {
         return "Show advanced command configuration in Command dialog";
@@ -338,12 +338,12 @@ struct show_advanced_command_settings : Config<bool> {
 };
 
 struct autocompletion : Config<bool> {
-    static QString name() { return "autocompletion"; }
+    static QString name() { return QStringLiteral("autocompletion"); }
     static Value defaultValue() { return true; }
 };
 
 struct max_process_manager_rows : Config<uint> {
-    static QString name() { return "max_process_manager_rows"; }
+    static QString name() { return QStringLiteral("max_process_manager_rows"); }
     static Value defaultValue() { return 1000; }
     static const char *description() {
         return "Maximum number of rows in process manager dialog";
@@ -351,7 +351,7 @@ struct max_process_manager_rows : Config<uint> {
 };
 
 struct save_delay_ms_on_item_added : Config<int> {
-    static QString name() { return "save_delay_ms_on_item_added"; }
+    static QString name() { return QStringLiteral("save_delay_ms_on_item_added"); }
     static Value defaultValue() { return 5 * 60 * 1000; }
     static const char *description() {
         return "Tab save delay after an item is added to the tab";
@@ -359,7 +359,7 @@ struct save_delay_ms_on_item_added : Config<int> {
 };
 
 struct save_delay_ms_on_item_modified : Config<int> {
-    static QString name() { return "save_delay_ms_on_item_modified"; }
+    static QString name() { return QStringLiteral("save_delay_ms_on_item_modified"); }
     static Value defaultValue() { return 5 * 60 * 1000; }
     static const char *description() {
         return "Tab save delay after an item is modified in the tab";
@@ -367,7 +367,7 @@ struct save_delay_ms_on_item_modified : Config<int> {
 };
 
 struct save_delay_ms_on_item_removed : Config<int> {
-    static QString name() { return "save_delay_ms_on_item_removed"; }
+    static QString name() { return QStringLiteral("save_delay_ms_on_item_removed"); }
     static Value defaultValue() { return 10 * 60 * 1000; }
     static const char *description() {
         return "Tab save delay after an item is removed from the tab";
@@ -375,7 +375,7 @@ struct save_delay_ms_on_item_removed : Config<int> {
 };
 
 struct save_delay_ms_on_item_moved : Config<int> {
-    static QString name() { return "save_delay_ms_on_item_moved"; }
+    static QString name() { return QStringLiteral("save_delay_ms_on_item_moved"); }
     static Value defaultValue() { return 30 * 60 * 1000; }
     static const char *description() {
         return "Tab save delay after an item is moved in the tab";
@@ -383,7 +383,7 @@ struct save_delay_ms_on_item_moved : Config<int> {
 };
 
 struct save_delay_ms_on_item_edited : Config<int> {
-    static QString name() { return "save_delay_ms_on_item_edited"; }
+    static QString name() { return QStringLiteral("save_delay_ms_on_item_edited"); }
     static Value defaultValue() { return 1000; }
     static const char *description() {
         return "Tab save delay after an item is edited in the tab";
@@ -391,7 +391,7 @@ struct save_delay_ms_on_item_edited : Config<int> {
 };
 
 struct save_on_app_deactivated : Config<bool> {
-    static QString name() { return "save_on_app_deactivated"; }
+    static QString name() { return QStringLiteral("save_on_app_deactivated"); }
     static Value defaultValue() { return true; }
     static const char *description() {
         return "Save unsaved tabs immediately after the app is deactivated (main window loses focus)";
@@ -399,7 +399,7 @@ struct save_on_app_deactivated : Config<bool> {
 };
 
 struct native_menu_bar : Config<bool> {
-    static QString name() { return "native_menu_bar"; }
+    static QString name() { return QStringLiteral("native_menu_bar"); }
 #ifdef Q_OS_MAC
     // Native menu bar doesn't show on macOS.
     // See: https://github.com/hluk/CopyQ/issues/1444
@@ -413,7 +413,7 @@ struct native_menu_bar : Config<bool> {
 };
 
 struct native_tray_menu : Config<bool> {
-    static QString name() { return "native_tray_menu"; }
+    static QString name() { return QStringLiteral("native_tray_menu"); }
     static Value defaultValue() { return false; }
     static const char *description() {
         return "Use native tray menu (menu items search may not work)";
@@ -421,7 +421,7 @@ struct native_tray_menu : Config<bool> {
 };
 
 struct script_paste_delay_ms : Config<int> {
-    static QString name() { return "script_paste_delay_ms"; }
+    static QString name() { return QStringLiteral("script_paste_delay_ms"); }
     static Value defaultValue() { return 250; }
     static const char *description() {
         return "Delay after pasting from script (when using `paste()`)";
@@ -429,7 +429,7 @@ struct script_paste_delay_ms : Config<int> {
 };
 
 struct window_paste_with_ctrl_v_regex : Config<QString> {
-    static QString name() { return "window_paste_with_ctrl_v_regex"; }
+    static QString name() { return QStringLiteral("window_paste_with_ctrl_v_regex"); }
     static const char *description() {
         return "Regular expression matching window titles where Ctrl+V should be used for pasting"
                " instead of the default Shift+Insert";
@@ -437,7 +437,7 @@ struct window_paste_with_ctrl_v_regex : Config<QString> {
 };
 
 struct window_wait_before_raise_ms : Config<int> {
-    static QString name() { return "window_wait_before_raise_ms"; }
+    static QString name() { return QStringLiteral("window_wait_before_raise_ms"); }
 #ifdef Q_OS_WIN
     static Value defaultValue() { return 0; }
 #else
@@ -449,7 +449,7 @@ struct window_wait_before_raise_ms : Config<int> {
 };
 
 struct window_wait_raised_ms : Config<int> {
-    static QString name() { return "window_wait_raised_ms"; }
+    static QString name() { return QStringLiteral("window_wait_raised_ms"); }
     static Value defaultValue() { return 150; }
     static const char *description() {
         return "Wait interval for raising target window for pasting";
@@ -457,7 +457,7 @@ struct window_wait_raised_ms : Config<int> {
 };
 
 struct window_wait_after_raised_ms : Config<int> {
-    static QString name() { return "window_wait_after_raised_ms"; }
+    static QString name() { return QStringLiteral("window_wait_after_raised_ms"); }
 #ifdef Q_OS_WIN
     static Value defaultValue() { return 150; }
 #elif defined(Q_OS_MAC)
@@ -471,7 +471,7 @@ struct window_wait_after_raised_ms : Config<int> {
 };
 
 struct window_key_press_time_ms : Config<int> {
-    static QString name() { return "window_key_press_time_ms"; }
+    static QString name() { return QStringLiteral("window_key_press_time_ms"); }
 #ifdef Q_OS_WIN
     static Value defaultValue() { return 0; }
 #elif defined(Q_OS_MAC)
@@ -486,7 +486,7 @@ struct window_key_press_time_ms : Config<int> {
 };
 
 struct window_wait_for_modifier_released_ms : Config<int> {
-    static QString name() { return "window_wait_for_modifier_released_ms"; }
+    static QString name() { return QStringLiteral("window_wait_for_modifier_released_ms"); }
     static Value defaultValue() { return 2000; }
     static const char *description() {
         return "Wait for any keyboard modifiers to be released before simulating/sending"
@@ -495,7 +495,7 @@ struct window_wait_for_modifier_released_ms : Config<int> {
 };
 
 struct update_clipboard_owner_delay_ms : Config<int> {
-    static QString name() { return "update_clipboard_owner_delay_ms"; }
+    static QString name() { return QStringLiteral("update_clipboard_owner_delay_ms"); }
     static Value defaultValue() { return -1; }
     static const char *description() {
         return "Delay to update new clipboard owner window title"
@@ -504,7 +504,7 @@ struct update_clipboard_owner_delay_ms : Config<int> {
 };
 
 struct row_index_from_one : Config<bool> {
-    static QString name() { return "row_index_from_one"; }
+    static QString name() { return QStringLiteral("row_index_from_one"); }
     static Value defaultValue() { return true; }
     static const char *description() {
         return "Index items/rows in UI starting from 1 instead of 0"
@@ -513,14 +513,14 @@ struct row_index_from_one : Config<bool> {
 };
 
 struct style : Config<QString> {
-    static QString name() { return "style"; }
+    static QString name() { return QStringLiteral("style"); }
     static const char *description() {
         return "Current application style (available styles can be listed with `copyq styles` command)";
     }
 };
 
 struct native_notifications : Config<bool> {
-    static QString name() { return "native_notifications"; }
+    static QString name() { return QStringLiteral("native_notifications"); }
     static Value defaultValue() { return true; }
 };
 
