@@ -5,7 +5,6 @@
 
 #include "common/clipboardmode.h"
 #include "gui/clipboardbrowsershared.h"
-#include "gui/theme.h"
 #include "item/clipboardmodel.h"
 #include "item/itemdelegate.h"
 #include "item/itemfilter.h"
@@ -17,8 +16,6 @@
 #include <QVariantMap>
 #include <QVector>
 
-#include <memory>
-
 class ItemEditorWidget;
 class ItemFactory;
 class PersistentDisplayItem;
@@ -102,7 +99,7 @@ class ClipboardBrowser final : public QListView
 
         void removeIndexes(const QModelIndexList &indexes, QString *error = nullptr);
 
-        bool canRemoveItems(const QModelIndexList &indexes, QString *error = nullptr) const;
+        bool canRemoveItems(const QModelIndexList &indexes, QString *error = nullptr);
 
         /** Render preview image with items. */
         QPixmap renderItemPreview(const QModelIndexList &indexes, int maxWidth, int maxHeight);
@@ -125,6 +122,8 @@ class ClipboardBrowser final : public QListView
                 int row = 0 //!< Target row for the new item (negative to append item).
                 );
 
+        bool addReversed(const QVector<QVariantMap> &dataList, int row);
+
         bool addAndSelect(const QVariantMap &data, int row);
 
         /**
@@ -132,6 +131,8 @@ class ClipboardBrowser final : public QListView
          */
         void addUnique(const QVariantMap &data, ClipboardMode mode);
 
+        void setItemsData(const QMap<QPersistentModelIndex, QVariantMap> &itemsData);
+
         /** Number of items in list. */
         int length() const { return m.rowCount(); }
 
@@ -144,9 +145,9 @@ class ClipboardBrowser final : public QListView
         /** Show only items matching the regular expression. */
         void filterItems(const ItemFilterPtr &filter);
         /** Open editor. */
-        bool openEditor(const QByteArray &textData, bool changeClipboard = false);
+        bool openEditor(const QModelIndex &index, const QString &format, const QByteArray &content, bool changeClipboard = false);
         /** Open editor for an item. */
-        bool openEditor(const QModelIndex &index);
+        bool openEditor(const QModelIndex &index, const QString &format);
 
         /** Set current item. */
         void setCurrent(int row, bool keepSelection = false, bool setCurrentOnly = false);
@@ -160,12 +161,10 @@ class ClipboardBrowser final : public QListView
          * Create and edit new item.
          */
         void editNew(
-                const QString &text = QString(), //!< Text of new item.
-                bool changeClipboard = false //!< Change clipboard if item is modified.
-                );
+            const QString &format, const QByteArray &content = {}, bool changeClipboard = false);
 
         /** Edit item in given @a row. */
-        void editRow(int row);
+        void editRow(int row, const QString &format);
 
         void move(int key);
 
@@ -189,8 +188,6 @@ class ClipboardBrowser final : public QListView
          */
         bool isFiltered(int row) const;
 
-        QVariantMap itemData(const QModelIndex &index) const;
-
         bool isLoaded() const;
 
         /**
@@ -226,6 +223,8 @@ class ClipboardBrowser final : public QListView
         using QListView::verticalOffset;
 
     signals:
+        void runOnRemoveItemsHandler(const QList<QPersistentModelIndex> &indexes, bool *canRemove);
+
         /** Show list request. */
         void requestShow(const ClipboardBrowser *self);
         /** Request clipboard change. */
@@ -330,7 +329,7 @@ class ClipboardBrowser final : public QListView
 
         void setEditorWidget(ItemEditorWidget *editor, bool changeClipboard = false);
 
-        void editItem(const QModelIndex &index, bool editNotes = false, bool changeClipboard = false);
+        void editItem(const QModelIndex &index, const QString &format, bool changeClipboard = false);
 
         void updateEditorGeometry();
 
@@ -354,6 +353,7 @@ class ClipboardBrowser final : public QListView
 
         /// Removes indexes without notifying or asking plugins.
         void dropIndexes(const QModelIndexList &indexes);
+        void dropIndexes(const QList<QPersistentModelIndex> &indexes);
 
         void focusEditedIndex();
 
