@@ -3,6 +3,7 @@
 
 class QPoint;
 class QRect;
+class QWidget;
 
 int screenCount();
 
@@ -10,6 +11,6 @@ int screenNumberAt(const QPoint &pos);
 
 QRect screenGeometry(int i);
 
-QRect screenAvailableGeometry(const QPoint &pos);
+QRect screenAvailableGeometry(const QWidget &w);
 
 #endif // SCREEN_H
