@@ -2,7 +2,10 @@
 
 #pragma once
 
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wreserved-identifier"
 typedef struct _XDisplay Display;
+#pragma clang diagnostic pop
 
 class X11Info {
 public:
