@@ -214,6 +214,7 @@ public slots:
     void insert();
     QJSValue remove();
     void edit();
+    QJSValue editItem();
     QJSValue move();
 
     QJSValue read();
@@ -427,12 +428,13 @@ private:
     QJSValue copy(ClipboardMode mode);
     QJSValue changeItem(bool create);
     void nextToClipboard(int where);
+    void editContent(int editRow, const QString &format, const QByteArray &content, bool changeClipboard);
     QJSValue screenshot(bool select);
     QByteArray serialize(const QJSValue &value);
     QJSValue eval(const QString &script);
     bool runAction(Action *action);
     bool runCommands(CommandType::CommandType type);
-    bool canExecuteCommand(const Command &command);
+    bool canExecuteCommand(const Command &command, QStringList *arguments);
     bool canExecuteCommandFilter(const QString &matchCommand);
     bool verifyClipboardAccess();
     void provideClipboard(ClipboardMode mode);
