@@ -13,7 +13,6 @@
 #include "common/display.h"
 #include "common/log.h"
 #include "common/mimetypes.h"
-#include "common/settings.h"
 #include "common/sleeptimer.h"
 #include "common/textdata.h"
 #include "gui/clipboardbrowser.h"
@@ -24,6 +23,7 @@
 #include "gui/notification.h"
 #include "gui/pixelratio.h"
 #include "gui/screen.h"
+#include "gui/selectiondata.h"
 #include "gui/tabicons.h"
 #include "gui/traymenu.h"
 #include "gui/windowgeometryguard.h"
@@ -88,7 +88,6 @@ void registerMetaTypes() {
     qRegisterMetaTypeStreamOperators<Command>("Command");
     qRegisterMetaTypeStreamOperators<NamedValueList>("NamedValueList");
     qRegisterMetaTypeStreamOperators<NotificationButtonList>("NotificationButtonList");
-    qRegisterMetaTypeStreamOperators<ScriptablePath>("ScriptablePath");
     qRegisterMetaTypeStreamOperators<QVector<int>>("QVector<int>");
     qRegisterMetaTypeStreamOperators<QVector<Command>>("QVector<Command>");
     qRegisterMetaTypeStreamOperators<VariantMapList>("VariantMapList");
@@ -99,7 +98,6 @@ void registerMetaTypes() {
     qRegisterMetaType<Command>("Command");
     qRegisterMetaType<NamedValueList>("NamedValueList");
     qRegisterMetaType<NotificationButtonList>("NotificationButtonList");
-    qRegisterMetaType<ScriptablePath>("ScriptablePath");
     qRegisterMetaType<QVector<int>>("QVector<int>");
     qRegisterMetaType<QVector<Command>>("QVector<Command>");
     qRegisterMetaType<VariantMapList>("VariantMapList");
@@ -260,16 +258,6 @@ QDataStream &operator>>(QDataStream &in, ClipboardMode &mode)
     return in;
 }
 
-QDataStream &operator<<(QDataStream &out, const ScriptablePath &path)
-{
-    return out << path.path;
-}
-
-QDataStream &operator>>(QDataStream &in, ScriptablePath &path)
-{
-    return in >> path.path;
-}
-
 QDataStream &operator<<(QDataStream &out, KeyboardModifierList value)
 {
     return out << static_cast<int>(value.items);
@@ -597,9 +585,9 @@ QWidget *createWidget(const QString &name, const QVariant &value, InputDialog *i
     case QVariant::StringList:
         return createListWidget(name, value.toStringList(), inputDialog);
     default:
-        if ( value.userType() == qMetaTypeId<ScriptablePath>() ) {
-            const auto path = value.value<ScriptablePath>();
-            return createFileNameEdit(name, path.path, parent);
+        if ( value.type() == QVariant::Url ) {
+            const auto path = value.toUrl();
+            return createFileNameEdit(name, path.toLocalFile(), parent);
         }
 
         const QString text = value.toString();
@@ -636,18 +624,6 @@ QString tabNameEmptyError()
     return ScriptableProxy::tr("Tab name cannot be empty!");
 }
 
-void raiseWindow(QPointer<QWidget> window)
-{
-    window->raise();
-    window->activateWindow();
-    QApplication::setActiveWindow(window);
-    QApplication::processEvents();
-    const auto wid = window->winId();
-    const auto platformWindow = platformNativeInterface()->getWindow(wid);
-    if (platformWindow)
-        platformWindow->raise();
-}
-
 } // namespace
 
 #ifdef HAS_TESTS
@@ -677,6 +653,7 @@ public:
         const auto currentWindow = platformNativeInterface()->getCurrentWindow();
         const auto currentWindowTitle = currentWindow ? currentWindow->getTitle() : QString();
         log( QString("Failed to send key press to target widget")
+            + QLatin1String(qApp->applicationState() == Qt::ApplicationActive ? "" : "\nApp is INACTIVE!")
             + "\nExpected: " + (expectedWidgetName.isEmpty() ? "Any" : expectedWidgetName)
             + "\nActual:   " + keyClicksTargetDescription(actual)
             + "\nPopup:    " + keyClicksTargetDescription(popup)
@@ -697,6 +674,16 @@ public:
             return;
         }
 
+        if (qApp->applicationState() != Qt::ApplicationActive) {
+#if defined(Q_OS_MAC) && QT_VERSION >= QT_VERSION_CHECK(6,0,0)
+            // WORKAROUND for focusing back to main window on macOS.
+            if (m_wnd->isVisible())
+                m_wnd->activateWindow();
+#endif
+            keyClicksRetry(expectedWidgetName, keys, delay, retry);
+            return;
+        }
+
         auto widgetName = keyClicksTargetDescription(widget);
         if ( !expectedWidgetName.isEmpty() && !widgetName.contains(expectedWidgetName) ) {
             keyClicksRetry(expectedWidgetName, keys, delay, retry);
@@ -1329,7 +1316,7 @@ QString ScriptableProxy::browserRemoveRows(const QString &tabName, QVector<int>
     INVOKE(browserRemoveRows, (tabName, rows));
     ClipboardBrowser *c = fetchBrowser(tabName);
     if (!c)
-        return QLatin1String("Invalid tab");
+        return QStringLiteral("Invalid tab");
 
     std::sort( rows.begin(), rows.end(), std::greater<int>() );
 
@@ -1355,30 +1342,29 @@ void ScriptableProxy::browserMoveSelected(int targetRow)
 {
     INVOKE2(browserMoveSelected, (targetRow));
 
-    const QList<QPersistentModelIndex> selected = selectedIndexes();
-    if ( selected.isEmpty() )
-        return;
-
-    ClipboardBrowser *c = m_wnd->browserForItem(selected.first());
+    QList<QPersistentModelIndex> selected = selectedIndexes();
+    selectionRemoveInvalid(&selected);
+    ClipboardBrowser *c = browserForIndexes(selected);
     if (c == nullptr)
         return;
 
     QModelIndexList indexes;
+    indexes.reserve(selected.size());
     for (const auto &index : selected)
         indexes.append(index);
     c->move(indexes, targetRow);
 }
 
-void ScriptableProxy::browserEditRow(const QString &tabName, int arg1)
+void ScriptableProxy::browserEditRow(const QString &tabName, int row, const QString &format)
 {
-    INVOKE2(browserEditRow, (tabName, arg1));
-    BROWSER(tabName, editRow(arg1));
+    INVOKE2(browserEditRow, (tabName, row, format));
+    BROWSER(tabName, editRow(row, format));
 }
 
-void ScriptableProxy::browserEditNew(const QString &tabName, const QString &arg1, bool changeClipboard)
+void ScriptableProxy::browserEditNew(const QString &tabName, const QString &format, const QByteArray &content, bool changeClipboard)
 {
-    INVOKE2(browserEditNew, (tabName, arg1, changeClipboard));
-    BROWSER(tabName, editNew(arg1, changeClipboard));
+    INVOKE2(browserEditNew, (tabName, format, content, changeClipboard));
+    BROWSER(tabName, editNew(format, content, changeClipboard));
 }
 
 QStringList ScriptableProxy::tabs()
@@ -1512,11 +1498,16 @@ int ScriptableProxy::browserLength(const QString &tabName)
     return c ? c->length() : 0;
 }
 
-bool ScriptableProxy::browserOpenEditor(const QString &tabName, const QByteArray &arg1, bool changeClipboard)
+bool ScriptableProxy::browserOpenEditor(
+    const QString &tabName, int row, const QString &format, const QByteArray &content, bool changeClipboard)
 {
-    INVOKE(browserOpenEditor, (tabName, arg1, changeClipboard));
+    INVOKE(browserOpenEditor, (tabName, row, format, content, changeClipboard));
     ClipboardBrowser *c = fetchBrowser(tabName);
-    return c && c->openEditor(arg1, changeClipboard);
+    if (!c)
+        return false;
+
+    const auto index = c->index(row);
+    return c->openEditor(index, format, content, changeClipboard);
 }
 
 QString ScriptableProxy::browserInsert(const QString &tabName, int row, const VariantMapList &items)
@@ -1525,15 +1516,13 @@ QString ScriptableProxy::browserInsert(const QString &tabName, int row, const Va
 
     ClipboardBrowser *c = fetchBrowser(tabName);
     if (!c)
-        return QLatin1String("Invalid tab");
+        return QStringLiteral("Invalid tab");
 
     if ( !c->allocateSpaceForNewItems(items.items.size()) )
-        return QLatin1String("Tab is full (cannot remove any items)");
+        return QStringLiteral("Tab is full (cannot remove any items)");
 
-    for (const auto &item : items.items) {
-        if ( !c->add(item, row) )
-            return QLatin1String("Failed to new add items");
-    }
+    if ( !c->addReversed(items.items, row) )
+        return QStringLiteral("Failed to add items");
 
     return QString();
 }
@@ -1544,21 +1533,24 @@ QString ScriptableProxy::browserChange(const QString &tabName, int row, const Va
 
     ClipboardBrowser *c = fetchBrowser(tabName);
     if (!c)
-        return QLatin1String("Invalid tab");
+        return QStringLiteral("Invalid tab");
+
+    QMap<QPersistentModelIndex, QVariantMap> itemsData;
 
     int currentRow = row;
     for (const auto &data : items.items) {
         const auto index = c->index(currentRow);
-        QVariantMap itemData = c->model()->data(index, contentType::data).toMap();
+        QVariantMap itemData = index.data(contentType::data).toMap();
         for (auto it = data.constBegin(); it != data.constEnd(); ++it) {
             if ( it.value().isValid() )
                 itemData.insert( it.key(), it.value() );
             else
                 itemData.remove( it.key() );
         }
-        c->model()->setData(index, itemData, contentType::data);
+        itemsData[index] = itemData;
         ++currentRow;
     }
+    c->setItemsData(itemsData);
 
     return QString();
 }
@@ -1592,8 +1584,7 @@ int ScriptableProxy::currentItem()
 {
     INVOKE(currentItem, ());
 
-    const QPersistentModelIndex current =
-            m_actionData.value(mimeCurrentItem).value<QPersistentModelIndex>();
+    const QPersistentModelIndex current = currentIndex();
     return current.isValid() ? current.row() : -1;
 }
 
@@ -1624,26 +1615,44 @@ QVector<int> ScriptableProxy::selectedItems()
     INVOKE(selectedItems, ());
 
     QVector<int> selectedRows;
-    const QList<QPersistentModelIndex> selected = selectedIndexes();
+    QList<QPersistentModelIndex> selected = selectedIndexes();
     selectedRows.reserve(selected.count());
-    for (const auto &index : selected) {
-        if (index.isValid())
-            selectedRows.append(index.row());
-    }
+
+    for (const auto &index : selected)
+        selectedRows.append(index.row());
 
     return selectedRows;
 }
 
+QString ScriptableProxy::selectedTab()
+{
+    INVOKE(selectedTab, ());
+
+    const QPersistentModelIndex current = currentIndex();
+    ClipboardBrowser *c = m_wnd->browserForItem(current);
+    if (c != nullptr)
+        return c->tabName();
+
+    const QList<QPersistentModelIndex> selected = selectedIndexes();
+    c = m_wnd->browserForItem(current);
+    if (c != nullptr)
+        return c->tabName();
+
+    return m_actionData.value(mimeCurrentTab).toString();
+}
+
 QVariantMap ScriptableProxy::selectedItemData(int selectedIndex)
 {
     INVOKE(selectedItemData, (selectedIndex));
 
-    auto c = currentBrowser();
-    if (!c)
-        return QVariantMap();
-
     const auto index = selectedIndexes().value(selectedIndex);
-    Q_ASSERT( !index.isValid() || index.model() == c->model() );
+    if ( !index.isValid() )
+        return {};
+
+    ClipboardBrowser *c = m_wnd->browserForItem(index);
+    if (c == nullptr)
+        return {};
+
     return c->copyIndex(index);
 }
 
@@ -1651,15 +1660,14 @@ bool ScriptableProxy::setSelectedItemData(int selectedIndex, const QVariantMap &
 {
     INVOKE(setSelectedItemData, (selectedIndex, data));
 
-    auto c = currentBrowser();
-    if (!c)
-        return false;
-
     const auto index = selectedIndexes().value(selectedIndex);
     if ( !index.isValid() )
         return false;
 
-    Q_ASSERT( index.model() == c->model() );
+    ClipboardBrowser *c = m_wnd->browserForItem(index);
+    if (c == nullptr)
+        return false;
+
     return c->model()->setData(index, data, contentType::data);
 }
 
@@ -1667,22 +1675,16 @@ VariantMapList ScriptableProxy::selectedItemsData()
 {
     INVOKE(selectedItemsData, ());
 
-    auto c = currentBrowser();
-    if (!c)
-        return {};
-
-    const auto model = c->model();
+    QList<QPersistentModelIndex> selected = selectedIndexes();
+    ClipboardBrowser *c = browserForIndexes(selected);
+    if (c == nullptr)
+        return {QVector<QVariantMap>(selected.size(), {})};
 
     QVector<QVariantMap> dataList;
-    const auto selected = selectedIndexes();
     dataList.reserve(selected.size());
 
-    for (const auto &index : selected) {
-        if ( index.isValid() ) {
-            Q_ASSERT( index.model() == model );
-            dataList.append( c->copyIndex(index) );
-        }
-    }
+    for (const auto &index : selected)
+        dataList.append( c->copyIndex(index) );
 
     return {dataList};
 }
@@ -1691,21 +1693,23 @@ void ScriptableProxy::setSelectedItemsData(const VariantMapList &dataList)
 {
     INVOKE2(setSelectedItemsData, (dataList));
 
-    auto c = currentBrowser();
-    if (!c)
-        return;
-
-    const auto model = c->model();
-
-    const auto indexes = selectedIndexes();
-    const auto count = std::min( indexes.size(), dataList.items.size() );
+    const QList<QPersistentModelIndex> selected = selectedIndexes();
+    ClipboardBrowser *c = nullptr;
+    QMap<QPersistentModelIndex, QVariantMap> itemsData;
+    const auto count = std::min( selected.size(), dataList.items.size() );
     for ( int i = 0; i < count; ++i ) {
-        const auto &index = indexes[i];
-        if ( index.isValid() ) {
-            Q_ASSERT( index.model() == model );
-            model->setData(index, dataList.items[i], contentType::data);
-        }
+        const auto &index = selected[i];
+        if ( !index.isValid() )
+            continue;
+
+        if (c == nullptr)
+            c = m_wnd->browserForItem(index);
+
+        itemsData[index] = dataList.items[i];
     }
+
+    if (c != nullptr)
+        c->setItemsData(itemsData);
 }
 
 int ScriptableProxy::createSelection(const QString &tabName)
@@ -1861,18 +1865,14 @@ void ScriptableProxy::selectionDeselectSelection(int id, int toDeselectId)
 void ScriptableProxy::selectionGetCurrent(int id)
 {
     INVOKE2(selectionGetCurrent, (id));
-    auto selection = m_selections.take(id);
-    if (!selection.browser)
-        return;
 
-    auto selected = selectedIndexes();
-    selectionRemoveInvalid(&selected);
-    if ( selected.isEmpty() ) {
-        m_selections[id] = {nullptr, {}};
-    } else {
-        ClipboardBrowser *c = m_wnd->browserForItem(selected.first());
-        m_selections[id] = {c, selected};
+    QList<QPersistentModelIndex> selected = selectedIndexes();
+    ClipboardBrowser *c = browserForIndexes(selected);
+    if (c == nullptr) {
+        const QString currentTab = m_actionData.value(mimeCurrentTab).toString();
+        c = fetchBrowser(currentTab);
     }
+    m_selections[id] = {c, selected};
 }
 
 int ScriptableProxy::selectionGetSize(int id)
@@ -1893,7 +1893,6 @@ QVector<int> ScriptableProxy::selectionGetRows(int id)
     INVOKE(selectionGetRows, (id));
 
     auto selection = m_selections.value(id);
-    selectionRemoveInvalid(&selection.indexes);
     QVector<int> rows;
     rows.reserve(selection.indexes.size());
     for (const auto &index : selection.indexes)
@@ -1941,13 +1940,16 @@ void ScriptableProxy::selectionSetItemsData(int id, const QVariantList &dataList
 {
     INVOKE2(selectionSetItemsData, (id, dataList));
 
+    QMap<QPersistentModelIndex, QVariantMap> itemsData;
     const auto selection = m_selections.value(id);
     const auto count = std::min( selection.indexes.size(), dataList.size() );
     for ( int i = 0; i < count; ++i ) {
         const auto &index = selection.indexes[i];
         if ( index.isValid() )
-            selection.browser->model()->setData(index, dataList[i], contentType::data);
+            itemsData[index] = dataList[i].toMap();
     }
+
+    selection.browser->setItemsData(itemsData);
 }
 
 QVariantList ScriptableProxy::selectionGetItemsFormat(int id, const QString &format)
@@ -1968,27 +1970,22 @@ void ScriptableProxy::selectionSetItemsFormat(int id, const QString &mime, const
     INVOKE2(selectionSetItemsFormat, (id, mime, value));
 
     const auto selection = m_selections.value(id);
-    for (const auto &index : selection.indexes) {
-        if ( index.isValid() ) {
-            QVariantMap data = index.data(contentType::data).toMap();
-            if (value.isValid())
-                data[mime] = value;
-            else
-                data.remove(mime);
-            selection.browser->model()->setData(index, data, contentType::data);
-        }
-    }
+    setItemsData(selection.browser, selection.indexes, mime, value);
 }
 
 void ScriptableProxy::selectionMove(int id, int row)
 {
     INVOKE2(selectionMove, (id, row));
     auto selection = m_selections.value(id);
-    selectionRemoveInvalid(&selection.indexes);
+    if (!selection.browser)
+        return;
 
     QModelIndexList indexes;
-    for (const auto &index : selection.indexes)
-        indexes.append(index);
+    indexes.reserve(selection.indexes.size());
+    for (const auto &index : selection.indexes) {
+        if (index.isValid())
+            indexes.append(index);
+    }
 
     if ( !indexes.isEmpty() )
         selection.browser->move(indexes, row);
@@ -2121,6 +2118,10 @@ int ScriptableProxy::inputDialog(const NamedValueList &values)
             widgets.append( createWidget(value.name, value.value, &inputDialog) );
     }
 
+    // WORKAROUND for broken initial focus in Qt 6.6 (QTBUG-121514)
+    if (!widgets.isEmpty())
+        widgets.first()->setFocus();
+
     dialog.adjustSize();
 
     if (geometry.height() == 0)
@@ -2163,6 +2164,8 @@ int ScriptableProxy::inputDialog(const NamedValueList &values)
                 const QVariant value = w->property(propertyName.toUtf8().constData());
                 result.items.append( NamedValue(name, value) );
             }
+            if ( widgets.isEmpty() )
+                result.items.append( NamedValue(QString(), true) );
         }
 
         QByteArray bytes;
@@ -2180,15 +2183,13 @@ int ScriptableProxy::inputDialog(const NamedValueList &values)
 
     if ( !dialogTitle.isNull() ) {
         dialog.setWindowTitle(dialogTitle);
-        dialog.setObjectName(QLatin1String("dialog_") + dialogTitle);
+        dialog.setObjectName(QStringLiteral("dialog_") + dialogTitle);
         WindowGeometryGuard::create(&dialog);
     }
 
     dialog.show();
 
-    // Skip raising dialog in tests.
-    if ( !qApp->property("CopyQ_test_id").isValid() )
-        raiseWindow(&dialog);
+    raiseWindow(&dialog);
 
     return dialogId;
 }
@@ -2196,18 +2197,14 @@ int ScriptableProxy::inputDialog(const NamedValueList &values)
 void ScriptableProxy::setSelectedItemsData(const QString &mime, const QVariant &value)
 {
     INVOKE2(setSelectedItemsData, (mime, value));
-    const QList<QPersistentModelIndex> selected = selectedIndexes();
-    for (const auto &index : selected) {
-        ClipboardBrowser *c = m_wnd->browserForItem(index);
-        if (c) {
-            QVariantMap data = c->model()->data(index, contentType::data).toMap();
-            if (value.isValid())
-                data[mime] = value;
-            else
-                data.remove(mime);
-            c->model()->setData(index, data, contentType::data);
-        }
-    }
+
+    QList<QPersistentModelIndex> selected = selectedIndexes();
+    selectionRemoveInvalid(&selected);
+    ClipboardBrowser *c = browserForIndexes(selected);
+    if (c == nullptr)
+        return;
+
+    setItemsData(c, selected, mime, value);
 }
 
 void ScriptableProxy::filter(const QString &text)
@@ -2601,6 +2598,12 @@ QStringList ScriptableProxy::styles()
     return QStyleFactory::keys();
 }
 
+void ScriptableProxy::setScriptOverrides(const QVector<int> &overrides)
+{
+    INVOKE2(setScriptOverrides, (overrides));
+    m_wnd->setScriptOverrides(overrides, m_actionId);
+}
+
 ClipboardBrowser *ScriptableProxy::fetchBrowser(const QString &tabName)
 {
     if (tabName.isEmpty()) {
@@ -2633,25 +2636,52 @@ QByteArray ScriptableProxy::itemData(const QString &tabName, int i, const QStrin
     return data.value(mime).toByteArray();
 }
 
-ClipboardBrowser *ScriptableProxy::currentBrowser() const
+void ScriptableProxy::setItemsData(
+    ClipboardBrowser *c, const QList<QPersistentModelIndex> &indexes, const QString &mime, const QVariant &value)
 {
-    const QString currentTabName = m_actionData.value(mimeCurrentTab).toString();
-    if (currentTabName.isEmpty())
-        return nullptr;
+    QMap<QPersistentModelIndex, QVariantMap> itemsData;
 
-    const int i = m_wnd->findTabIndex(currentTabName);
-    if (i == -1)
-        return nullptr;
+    for (const auto &index : indexes) {
+        if ( !index.isValid() )
+            continue;
 
-    auto c = m_wnd->browser(i);
-    Q_ASSERT(c->tabName() == currentTabName);
-    return c;
+        QVariantMap data = index.data(contentType::data).toMap();
+        if (value.isValid())
+            data[mime] = value;
+        else
+            data.remove(mime);
+        itemsData[index] = data;
+    }
+
+    c->setItemsData(itemsData);
 }
 
-QList<QPersistentModelIndex> ScriptableProxy::selectedIndexes() const
+template<typename T>
+T ScriptableProxy::getSelectionData(const QString &mime)
 {
-    return m_actionData.value(mimeSelectedItems)
-            .value< QList<QPersistentModelIndex> >();
+    QVariant value = m_actionData.value(mime);
+    if ( !value.isValid() && !m_actionData.contains(mimeCurrentTab) && getSelectionData() )
+        value = m_actionData.value(mime);
+    return value.value<T>();
+}
+
+QPersistentModelIndex ScriptableProxy::currentIndex()
+{
+    return getSelectionData<QPersistentModelIndex>(mimeCurrentItem);
+}
+
+QList<QPersistentModelIndex> ScriptableProxy::selectedIndexes()
+{
+    return getSelectionData<QList<QPersistentModelIndex>>(mimeSelectedItems);
+}
+
+ClipboardBrowser *ScriptableProxy::browserForIndexes(const QList<QPersistentModelIndex> &indexes) const
+{
+    for (const auto &index : indexes) {
+        if ( index.isValid() )
+            return m_wnd->browserForItem(index);
+    }
+    return nullptr;
 }
 
 QVariant ScriptableProxy::waitForFunctionCallFinished(int functionCallId)
@@ -2677,6 +2707,18 @@ QVariant ScriptableProxy::waitForFunctionCallFinished(int functionCallId)
     return result;
 }
 
+bool ScriptableProxy::getSelectionData()
+{
+    auto c = m_wnd->browser();
+    if (c == nullptr)
+        return false;
+
+    const QVariantMap data = selectionData(*c);
+    for (auto it = data.constBegin(); it != data.constEnd(); ++it)
+        m_actionData[it.key()] = it.value();
+    return true;
+}
+
 #ifdef HAS_TESTS
 KeyClicker *ScriptableProxy::keyClicker()
 {
@@ -2704,3 +2746,22 @@ QString translationsPath()
 {
     return platformNativeInterface()->translationPrefix();
 }
+
+void setClipboardMonitorRunning(bool running)
+{
+    QSettings settings(
+          QSettings::IniFormat,
+          QSettings::UserScope,
+          QCoreApplication::organizationName(),
+          QCoreApplication::applicationName() + "-monitor");
+    settings.setValue(QStringLiteral("running"), running);
+}
+bool isClipboardMonitorRunning()
+{
+    const QSettings settings(
+          QSettings::IniFormat,
+          QSettings::UserScope,
+          QCoreApplication::organizationName(),
+          QCoreApplication::applicationName() + "-monitor");
+    return settings.value(QStringLiteral("running")).toBool();
+}
